﻿using File_Watcher.Models;
using Microsoft.Extensions.Hosting.WindowsServices;

namespace File_Watcher;

public partial class Worker : BackgroundService
{

    private readonly bool _IsWindowsService;
    private readonly ILogger<Worker> _Logger;
    private readonly AppSettings _AppSettings;

    public Worker(IServiceProvider serviceProvider, ILogger<Worker> logger, AppSettings appSettings, List<string> collection)
    {
        _Logger = logger;
        _AppSettings = appSettings;
        logger.LogInformation("{buildNumber}-{gitCommitSeven}", _AppSettings.BuildNumber, _AppSettings.GitCommitSeven);
        try
        { logger.LogInformation("<{folder}>", Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData)); }
        catch (Exception) { }
        _IsWindowsService = collection.Contains(nameof(WindowsServiceLifetime));
    }

    private void Body()
    {
        _Logger.LogInformation("A) Next execute will be at {date}", DateTime.Now.AddMilliseconds(_AppSettings.MillisecondsDelay).ToString("yyyy-MM-dd hh:mm:ss.fff tt"));
        if (!Directory.Exists(_AppSettings.WatchDirectory))
            _ = Directory.CreateDirectory(_AppSettings.WatchDirectory);
        else
        {
            _ = _AppSettings.Helper switch
            {
                nameof(Helpers.HelperEAFLog) => Helpers.HelperEAFLog.DeleteFiles(_AppSettings, _Logger),
                nameof(Helpers.HelperStratus) => Helpers.HelperStratus.MoveFile(_AppSettings, _Logger),
                nameof(Helpers.HelperWaferCounter) => Helpers.HelperWaferCounter.MoveFile(_AppSettings, _Logger),
                _ => throw new NotSupportedException()
            };
        }
        _Logger.LogInformation("B) Next execute will be at {date}", DateTime.Now.AddMilliseconds(_AppSettings.MillisecondsDelay).ToString("yyyy-MM-dd hh:mm:ss.fff tt"));
    }

    private async Task Body(CancellationToken stoppingToken)
    {
        while (_IsWindowsService && !stoppingToken.IsCancellationRequested)
        {
            Body();
            await Task.Delay(_AppSettings.MillisecondsDelay, stoppingToken);
        }
    }

    protected override async Task ExecuteAsync(CancellationToken stoppingToken) =>
        await Body(stoppingToken);
}